<?php

namespace Drupal\feeds_tamper_media_url\Plugin\Tamper;

use Drupal\Core\Form\FormStateInterface;
use Drupal\tamper\Exception\TamperException;
use Drupal\tamper\TamperableItemInterface;
use Drupal\tamper\TamperBase;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Entity\EntityStorageException;
use GuzzleHttp\Client;
use Drupal\Core\File\Exception\FileException;
use Drupal\media\Entity\Media;
use Drupal\media\Entity\MediaType;
use Drupal\field\Entity\FieldConfig;
use Drupal\field\Entity\FieldStorageConfig;
use \Drupal\Core\File\FileExists;


/**
 * Plugin implementation of the find_replace plugin.
 *
 * @Tamper(
 *   id = "create_media_tamper",
 *   label = @Translation("Create Media Tamper"),
 *   description = @Translation("Create Media Tamper"),
 *   category = "Other"
 * )
 */
class CreateMedia extends TamperBase
{

  const SETTING_MEDIA_TYPE = 'media_type';
  const SETTING_MEDIA_FIELD = 'media_field';

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration()
  {
    $config = parent::defaultConfiguration();
    $config[self::SETTING_MEDIA_TYPE] = '';
    $config[self::SETTING_MEDIA_FIELD] = '';
    return $config;
  }


  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state)
  {
    $media_types = MediaType::loadMultiple();
    $options = [];

    foreach ($media_types as $media_type) {
      $options[$media_type->id()] = $media_type->label();
    }
    $form[self::SETTING_MEDIA_TYPE] = [
      '#type' => 'select',
      '#title' => $this->t('Select Media Types'),
      '#default_value' => $this->getSetting(self::SETTING_MEDIA_TYPE),
      '#options' => $options,
    ];

    $form[self::SETTING_MEDIA_FIELD] = [
      '#type' => 'textfield',
      '#title' => $this->t('Whats the media field?'),
      '#default_value' => $this->getSetting(self::SETTING_MEDIA_FIELD),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state)
  {
    parent::submitConfigurationForm($form, $form_state);
    $this->setConfiguration([
      self::SETTING_MEDIA_TYPE => $form_state->getValue(self::SETTING_MEDIA_TYPE),
      self::SETTING_MEDIA_FIELD => $form_state->getValue(self::SETTING_MEDIA_FIELD),
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function tamper($data, TamperableItemInterface $item = NULL)
  {
    if (empty($data)) {
      // No file.
      return $data;
    }

    //Remove URL potential white spaces
    $data = preg_replace('/\s+/', '', $data);
    
    $fileSystem = \Drupal::service('file_system');
    $token_service = \Drupal::token();
    $media_type = $this->getSetting(self::SETTING_MEDIA_TYPE);
    $media_field = $this->getSetting(self::SETTING_MEDIA_FIELD);
    $field_definition = \Drupal::service('entity_field.manager')->getFieldDefinitions('media', $media_type)[$media_field];
    $file_directory =  $token_service->replace($field_definition->getSetting('file_directory'));

    $file_name = $this->getFileName($fileSystem, $data);
    // Perform a lookup against the value using the configured reference method.
    $file = $media_type !== "remote_video" ? $this->findFile($file_name) : $data;
    $filepath = 'public://' . $file_directory . '/'  . $file_name;
    if(!$this->checkFileAccessibility($filepath, $media_type) ||  !isset($file)){
      $file_content = $this->getContent($data);
      if(!$file_content){
        return '';
      }
      $file = $this->writeData($file_content,  $file_name , $fileSystem ,$filepath,FileExists::Replace );
    }


    // Compose file path.
    // $filepath = $this->getDestinationDirectory($fileSystem) . '/' . $file_name;

    if ($file) {
      $media_id = $file instanceof \Drupal\file\Entity\File ? $file->id() : $file;
      $media = $this->findMedia($media_id, $media_field);
      if (!$media) {
        $media = Media::create([
          'name' => $file_name,
          'bundle' => $media_type,
          'uid' => 1,
          'langcode' => 'en',
          'status' => 1,
          $media_field => [
            'target_id' => $media_id,
            'alt' => $file_name,
            'title' => $file_name,
          ],
        ]);
        $media->save();
      }

      return $media->id();
    }
  }

  protected function getFileName($fileSystem, $url)
  {
    $filename = trim($fileSystem->basename($url), " \t\n\r\0\x0B.");
    // Remove query string from file name, if it has one.
    [$filename] = explode('?', $filename);
    // $extension = substr($filename, strrpos($filename, '.') + 1);


    return  $filename;
  }


  protected function getContent($url)
  {
    $client = new Client();
    try {
      $response = $client->request('GET', $url);
      if ($response->getStatusCode() >= 400) {
        $args = [
          '%url' => $url,
          '@code' => $response->getStatusCode(),
        ];
        return false;
      }
  
      return (string) $response->getBody();
    } catch (\Throwable $e) {
      return false;
    }
    


  }

  protected function writeData($data, $file_name, $fileSystem,  $destination = NULL, $replace = FileExists::Replace)
  {
    $fileConfig = \Drupal::config('file.settings');
    $fileRepository = \Drupal::service('file.repository');
    if (empty($destination)) {
      $destination = $fileConfig->get('default_scheme') . '://';
    }
    try {
      $destination = str_replace($file_name, '',$destination);
      $fileSystem->prepareDirectory($destination, FileSystemInterface::CREATE_DIRECTORY);
      return $fileRepository->writeData($data, $destination . $file_name, $replace);
    } catch (EntityStorageException | FileException $e) {
      return FALSE;
    }
  }

  protected function findFile(string $file_name)
  {
    // Check if a file with the same name already exists.
    $existing_files = \Drupal::entityTypeManager()
      ->getStorage('file')
      ->loadByProperties(['filename' => $file_name]);

    if (!empty($existing_files)) {
      // File already exists.
      return last($existing_files);
    }
    return FALSE;
  }

  protected function findMedia($fid, $media_field)
  {
    // Check if a file with the same name already exists.
    $existing_media = \Drupal::entityTypeManager()
      ->getStorage('media')
      ->loadByProperties([$media_field => $fid]);

    if (!empty($existing_media)) {
      // File already exists.
      return reset($existing_media);
    }
    return FALSE;
  }

  protected function checkFileAccessibility($filePath, $media_type) {
    // First check if the file exists in the file system.
    if (file_exists($filePath)) {
        // If it exists, get the URL of the file.
        $fileUrl = \Drupal::request()->getSchemeAndHttpHost() . str_replace('public://', '', $filePath);
        
        $client = new Client();
        try {
          $response = $client->request('GET', $fileUrl);
   
          return TRUE;
        } catch (\Throwable $e) {
          \Drupal::service('file_system')->delete($filePath);
          return FALSE;        }
        // Use get_headers to check the HTTP response code.
    } elseif ($media_type == "remote_video") {
        return TRUE;
    } else {
      return FALSE;
    }
}
}
